## no critic qw(Capitalization ProhibitMultiplePackages ProhibitReusedNames)  # SYSTEM DEFAULT 3: allow multiple & lower case package names
package rperlnamespaces_generated;
use strict;
use warnings;
our $VERSION = 0.001_000;

## no critic qw(ProhibitParensWithBuiltins ProhibitNoisyQuotes)  # SYSTEM SPECIAL 3: allow auto-generated code

$rperlnamespaces_generated::CORE = undef;
$rperlnamespaces_generated::CORE = {
    '!'             => 1,
    'B::'           => 1,
    'CORE::'        => 1,
    'Carp::'        => 1,
    'DB::'          => 1,
    'Data::'        => 1,
    'DynaLoader::'  => 1,
    'ENV'           => 1,
    'ERRNO'         => 1,
    'English::'     => 1,
    'Errno::'       => 1,
    'Exporter::'    => 1,
    'INC'           => 1,
    'IO::'          => 1,
    'Internals::'   => 1,
    'OS_ERROR'      => 1,
    'PerlIO::'      => 1,
    'Regexp::'      => 1,
    'SIG'           => 1,
    'Scalar::'      => 1,
    'Tie::'         => 1,
    'UNIVERSAL::'   => 1,
    'XSLoader::'    => 1,
    'bytes::'       => 1,
    'constant::'    => 1,
    'main::'        => 1,
    'mro::'         => 1,
    'overload::'    => 1,
    'overloading::' => 1,
    're::'          => 1,
    'strict::'      => 1,
    'threads::'     => 1,
    'utf8::'        => 1,
    'vars::'        => 1,
    'version::'     => 1,
    'warnings::'    => 1
};

$rperlnamespaces_generated::RPERL = undef;
$rperlnamespaces_generated::RPERL = {
    'Config::'                       => 1,
    'Cwd::'                          => 1,
    'Digest::'                       => 1,
    'Dos::'                          => 1,
    'EPOC::'                         => 1,
    'ExtUtils::'                     => 1,
    'Fcntl::'                        => 1,
    'File::'                         => 1,
    'FindBin::'                      => 1,
    'Inline::'                       => 1,
    'List::'                         => 1,
    'MIME::'                         => 1,
    'POSIX::'                        => 1,
    'PadWalker::'                    => 1,
    'RPerl::'                        => 1,
    'Socket::'                       => 1,
    'VMS::'                          => 1,
    'Win32::'                        => 1,
    'array::'                        => 1,
    'arrayref::'                     => 1,
    'arrayref_arrayref::'            => 1,
    'arrayref_arrayref_method::'     => 1,
    'arrayref_hashref::'             => 1,
    'bool::'                         => 1,
    'char::'                         => 1,
    'char_arrayref::'                => 1,
    'char_hashref::'                 => 1,
    'char_method::'                  => 1,
    'coderef::'                      => 1,
    'filehandleref::'                => 1,
    'float::'                        => 1,
    'float_arrayref::'               => 1,
    'float_hashref::'                => 1,
    'float_method::'                 => 1,
    'hash::'                         => 1,
    'hashref::'                      => 1,
    'hashref_arrayref::'             => 1,
    'hashref_hashref::'              => 1,
    'hashref_hashref_method::'       => 1,
    'hashref_method::'               => 1,
    'integer::'                      => 1,
    'integer_arrayref::'             => 1,
    'integer_arrayref_arrayref::'    => 1,
    'integer_arrayref_method::'      => 1,
    'integer_hashref::'              => 1,
    'integer_hashref_hashref::'      => 1,
    'integer_hashref_method::'       => 1,
    'integer_method::'               => 1,
    'method::'                       => 1,
    'number::'                       => 1,
    'number_arrayref::'              => 1,
    'number_arrayref_arrayref::'     => 1,
    'number_arrayref_method::'       => 1,
    'number_hashref::'               => 1,
    'number_hashref_hashref::'       => 1,
    'number_hashref_method::'        => 1,
    'number_method::'                => 1,
    'object::'                       => 1,
    'object_arrayref::'              => 1,
    'object_arrayref_method::'       => 1,
    'object_hashref::'               => 1,
    'object_hashref_method::'        => 1,
    'object_method::'                => 1,
    'parent::'                       => 1,
    'ref::'                          => 1,
    'rperlnames::'                   => 1,
    'rperlnamespaces::'              => 1,
    'rperlnamespaces_generated::'    => 1,
    'rperltypes::'                   => 1,
    'scalartype::'                   => 1,
    'scalartype_arrayref::'          => 1,
    'scalartype_arrayref_arrayref::' => 1,
    'scalartype_hashref::'           => 1,
    'scalartype_hashref_hashref::'   => 1,
    'scalartype_method::'            => 1,
    'string::'                       => 1,
    'string_arrayref::'              => 1,
    'string_arrayref_arrayref::'     => 1,
    'string_arrayref_method::'       => 1,
    'string_hashref::'               => 1,
    'string_hashref_hashref::'       => 1,
    'string_hashref_method::'        => 1,
    'string_method::'                => 1,
    'unknown::'                      => 1,
    'unknown_method::'               => 1,
    'void::'                         => 1,
    'void_method::'                  => 1
};

1;
