package RPerl::GrammarRule;
use strict;
use warnings;
use RPerl;
our $VERSION = 0.000_002;

use parent qw(RPerl::CompileUnit::Module::Class); # inherit directly from base class
our hashref $properties = {};

1;
